/*
    PROGRAM NAME: SANDSLIDER CONTROLLER

    DESCRIPTION : This is the program for Arduino nano to control the sand slicer device.
                  The device is used to extrude and slice a cylinder of magic sand out of a syringe.
                  Each sand section is automatically photographed by an iPhone controlled by a Bluetooth
                  shutter.
                  The device is commanded and controlled by an Arduino nano using this program.
                  The joystick module is used to drive a menu on a LCD 20x2, set up the parameters used by the
                  slicer and to run the slicing sequence.

                  The Y-positions of the joystick is used to select the menu option, the joystick
                  button to select/confirm and the X-positions to assign values to the different menu options.

  LCD MENU DESCRIPTION:

                  1. # slides:        This option assigns the number of slides (default 1). If a negative value
                                      is given the slicer will run until it is stopped manually by pressing the
                                      joystick button.
                  2. # rotations:     This option set up the number of full rotations of the
                                      extruder step motor.
                     Slide thickness: It gives the approximate thickness calculated using
                                      the thickness converter parameter (variable rot2tick) set into
                                      the program code. By setting a negative   number of rotations,  the
                                      server will rotate in the opposite direction.
                  3. # blade cuts:    It set the number of time the blade will slide up/down. By repeating the slicing cuts, 
                                      it can help to make flatter the section surface by removing no uniform regions.
                  4. # brush swips:   It set the number of time the brush will swip the cut surface of the sand to clean 
                                      by cutting residues
                  5. Run it!          Run the slicing sequence. Pressing the joystick during the sequence  will
                                      stop/go the sequence. Pressing up after the sequence is stopped will bring back
                                      to the menu.
                  6. Test blade:      Test the blade movement.
                  7. Test brush:      Test brush movement.
                  8. Test extruder:   Test the extruder movement.
                  9. Adjust sensor:   It is use to test and adjust the position of the end of run IR sensor.
                 10. Reset:           Set to zero the counter of the number of slices.


    AUTHOR : Danilo Roccatano (c)
    VERSION: 2.2
    DATE   : January-July 2021
*/

// include Arduino stepper motor library
#include <Servo.h>
#include <Stepper.h>

// include Arduino LCD library
#include <Wire.h>
#include <LiquidCrystal_I2C.h>

//LCD settings
// Set the LCD address to 0x27 for a 16 chars and 2 line display
const uint8_t I2C_ADDRESS = 0x27; //watch video. use I2C Scanenr to get the address
const uint8_t LCD_CHAR = 16;
const uint8_t LCD_LINE = 2;

char TITLE_LCD[4][20] = {"The SAND SLICER", "v.1. D.Roccatano ", "# Slices:", "# Rotat.:"};
char MENU_TITLE[10][20] = {"1-# Slices:", "2-# Slice Thick:", "3-# B.Cuts:", "4-# B.Brush:", "5-Run it! ", "6-Test Blad&Shut", "7-Test Brush", "8-Test Extruder", "9-Test Sensor","10-Default Param"};


int nrot = 1;           // number of extruder rotation
int imenu = 0;          // select menu item
int nms = 10;          // number of microsteps x stepper rotation
int rinc = nms;
int nsl = 1;            // number of slices
int ncut = 3;           // number of blade cuts
int nbru = 1;           // number of brush swips
float rot2thick = 0.0;  // rotation to thickness conversion factor. This factor need to be and assigned estimated.
int buttonPin = 2;

int buttonState = 0;    // joystick button

int isObstaclePin = 5;  // This is used to check IR sensor input pin 5
int isObstacle = HIGH;  // HIGH MEANS NO OBSTACLE

LiquidCrystal_I2C lcd(I2C_ADDRESS, LCD_CHAR, LCD_LINE);

// Extruder stepper setup
// define total number of steps per revolution
#define STEPS 2038

unsigned int nsteps = STEPS / nms; //number of step per 360 degree rotation of the extruder stepper motor

// define extruder stepper motor control pins
#define IN1  11
#define IN2  10
#define IN3   9
#define IN4   8

// push button pin
//#define PBUTTON_PIN 7

// Joystick module setup
#define GROUND_JOY_PIN A3            //joystick ground pin will connect to Arduino analog pin A3
#define VOUT_JOY_PIN A2              //joystick +5 V pin will connect to Arduino analog pin A2
#define XJOY_PIN A1                  //X axis reading from joystick will go into analog pin A1
#define YJOY_PIN A0                  //Y axis reading from joystick will go into analog pin A1

// setup the servo object name for the blade movement
Servo myservo ; // slice bar control
// setup the servo object name for the iphone shutter control
Servo myservo1 ; // control the iphone
// setup the servo object name for the brush
Servo myservo2 ; 

// initialize stepper library
Stepper stepper(STEPS, IN4, IN2, IN3, IN1);

void setup()
{
  Serial.begin(9600);
  pinMode(VOUT_JOY_PIN, OUTPUT) ;    //pin A3 shall be used as output
  pinMode(GROUND_JOY_PIN, OUTPUT) ;  //pin A2 shall be used as output
  digitalWrite(VOUT_JOY_PIN, HIGH) ; //set pin A3 to high (+5V)
  digitalWrite(GROUND_JOY_PIN, LOW) ; //set pin A3 to low (ground)
  pinMode(buttonPin, INPUT_PULLUP);

  pinMode(isObstaclePin, INPUT);    // sensor pin set to input

  // setup the pins numbers of the servos for the blade and shutter
  myservo.attach(4);
  myservo1.attach(12);
  myservo2.attach(6);

  lcd.init();
  lcd.backlight();
  lcd.print(TITLE_LCD[0]);
  lcd.setCursor(0, 1);
  lcd.print(TITLE_LCD[1]);
  delay(2000);
  DisplayMenu(0);
  lcdDisplayLine2(nsl);
}

void loop()
{
  // Read the joystick button state
  buttonState = digitalRead(buttonPin);

  if (!buttonState) {

    switch (imenu) {
      case 4:
        // Run it!
        RunRotation (nrot, nsl, rinc);
        delay(300);
        buttonState = 0;
        break;
      case 5:
        // Test blade
         BladeShutterControl(ncut);
        break;
      case 6:
        // Test brush
         BrushControl(nbru);
        break;        
      case 7:
        // Test servo
        rinc = nms;
        if (nrot < 0) rinc = -nms;
        stepper.setSpeed(16); // 1 rpm
        stepper.step(rinc);
        break;
      case 8:
        // Test end of run sensor
        EoRSensor();
        break;        
      case 9:
        // reset parameters
        nsl = 1;
        nrot = 1;
        ncut = 3;
        nbru = 1;
    }
    DisplayMenu(imenu);
  }

  // read analog value from the X potentiometer

  int joystickXVal = analogRead(XJOY_PIN) ;  //read joystick input on pin A1

  if (  (joystickXVal > 350) && (joystickXVal < 550) )
  {
    digitalWrite(IN1, LOW);
    digitalWrite(IN2, LOW);
    digitalWrite(IN3, LOW);
    digitalWrite(IN4, LOW);
  }
  else
  {
     while (joystickXVal <= 350)
    {
      switch (imenu) {
        case 0:
          nsl++;
          lcdDisplayLine2(nsl);
          break;
        case 1:
          nrot++;
          lcdDisplayLine2(nrot);
          break;
        case 2:
          ncut++;
          lcdDisplayLine2(ncut);
          break;
        case 3:
          nbru++;
          lcdDisplayLine2(nbru);
      }
      delay(200);
      joystickXVal = analogRead(XJOY_PIN) ;  //read joystick input on pin A1
    }
    while (joystickXVal >= 550)
    {
      switch (imenu) {
        case 0:
          nsl--;
          if (nsl < 1) nsl = -1; // (-1) correspond to infinite slides
          lcdDisplayLine2(nsl);
          break;
        case 1:
          nrot--;
           if (nrot == 0) nrot = -1;
          lcdDisplayLine2(nrot);
          break;
        case 2:
          ncut--;
          if (ncut < 1) ncut = 1;
          lcdDisplayLine2(ncut);
          break;
        case 3:
          nbru--;
          if (nbru < 1) nbru = 1;
          lcdDisplayLine2(nbru);      
      }
      delay(200);
      joystickXVal = analogRead(XJOY_PIN) ;  //read joystick input on pin A1
    }
  }

  //
  // read analog value from the Y potentiometer and change the menu items on the LCD
  //

  int joystickYVal = analogRead(YJOY_PIN) ;  //read joystick input on pin A0
  if (  (joystickYVal > 350) && (joystickYVal < 550) )
  {
    digitalWrite(IN1, LOW);
    digitalWrite(IN2, LOW);
    digitalWrite(IN3, LOW);
    digitalWrite(IN4, LOW);
  }
  else
  {
    while (joystickYVal >= 550)
    {
      imenu++;
      if (imenu > 9) imenu = 0;
      DisplayMenu(imenu);
      delay(200);
      joystickYVal = analogRead(YJOY_PIN) ;  //read joystick input on pin A1
    }
    // Move menu in the other direction
    while (joystickYVal <= 350)
    {
      imenu--;
      if (imenu < 0) imenu = 9;
      DisplayMenu(imenu);
      delay(200);
      joystickYVal = analogRead(YJOY_PIN) ;  //read joystick input on pin A1
    }
  }
}

//
// FUNCTIONS
//
void DisplayMenu( int imenu) {

  lcdDisplayLine1(imenu);
  switch (imenu) {
    case 0:
      lcdDisplayLine2(nsl);
      break;
    case 1:
      lcdDisplayLine2(nrot);
      break;
    case 2:
      lcdDisplayLine2(ncut);
      break;
    case 3:
      lcdDisplayLine2(nbru);
      break;
    default:
      clearLine2();
  }
}

void RunRotation (int nrot, int nsl, int rinc) {
  int nr = 0;
  int ns = nsl;
  int nss = 1;
  lcd.init();
  lcd.backlight();
  lcd.setCursor(0, 0);
  lcd.print(TITLE_LCD[2]);
  lcd.setCursor(0, 1);
  lcd.print(TITLE_LCD[3]);
  delay(200);
  bool lct = 1;
  rinc = nms;
  if (nrot < 0) rinc = -nms;

  while (lct) {
    while (nr <= nrot * nsteps)
    {
      // Read the joystick button state
      buttonState = digitalRead(buttonPin);
      isObstacle = digitalRead(isObstaclePin);
      // Abort procedure when joystick button is pushed or the sensor detect end of the run
      if (!buttonState || isObstacle == HIGH ) {
        return;
      }
      
      stepper.setSpeed(16); // 1 rpm
      stepper.step(rinc);
      nr++;
      lcd.setCursor(10, 1);
      lcd.print(nr);  
      delay(10);
    }
    if (ns != -1) {
       lcd.setCursor(10, 0);
       lcd.print("           ");
       lcd.setCursor(10, 0);
       lcd.print(ns);
    } else {
       lcd.setCursor(10, 0);
       lcd.print("           ");
       lcd.setCursor(10, 0);
       lcd.print(nss);      
    }
    delay(10);
    if (ns != -1) {
      ns--;
      nr = 0;
      nss++;
      if (ns == 0) lct = 0;
      BladeShutterControl(ncut);
      BrushControl(nbru);
    } else {
      nr = 0;
      nss++;
      BladeShutterControl(ncut);
      BrushControl(nbru);
    }
  }
}

void BladeShutterControl(int ncut) {

  for (int i = 1; i <= ncut; i++) {
    myservo.write(180);
    delay(500);
    myservo.write(90);
    delay(500);
  }
}

void BrushControl(int nbru) {

  for (int i = 1; i <= nbru; i++) {
    myservo2.write(180);
    delay(500);
    myservo2.write(90);
    delay(500);
  }
  // Press the shutter for the photo
  myservo1.write(30);
  delay(500);
  myservo1.write(90);
}

void EoRSensor() {    
    // Abort procedure when joystick button is pushed 
    clearLine1();
    lcd.setCursor(0, 0);
    isObstacle = digitalRead(isObstaclePin);
       if (isObstacle == LOW)
       {
        clearLine1();
        lcd.setCursor(0, 0);
        lcd.print("CONTINUE...");
       }
      else 
      {
        clearLine1();
        lcd.setCursor(0, 0);
        lcd.print("END OF RUN!");
      }
      delay(1000);
  }


void lcdDisplay(int imenu)
{
  clearLine2();
  lcd.setCursor(0, 1);
  lcd.print(MENU_TITLE[imenu]);

} //

void lcdDisplayLine1(int imenu)
{
  clearLine1();
  lcd.setCursor(0, 0);
  lcd.print(MENU_TITLE[imenu]);

} //

void lcdDisplayLine2(int value)
{
  clearLine2();
  lcd.setCursor(0, 1);
  lcd.print(value);//print value
}


void clearLine1()
{
  for (int i = 0; i <= LCD_CHAR - 1; i++)
  {
    lcd.setCursor (i, 0);
    lcd.write(254);
  }
}

void clearLine2()
{
  for (int i = 0; i <= LCD_CHAR - 1; i++)
  {
    lcd.setCursor (i, 1); //
    lcd.write(254);
  }
}
